/* PTController
01:02:03:04

*/

#import <Cocoa/Cocoa.h>

extern NSString *PTCONTROLUseCommandKey;
extern NSString *PTCONTROLUseShiftKey;
extern NSString *PTCONTROLUseControlKey;
extern NSString *PTCONTROLUseOptionKey;
extern NSString *PTCONTROLKeyCode;
extern NSString *PTCONTROLCharCode;
extern NSString *PTCONTROLKeyDescription;
extern NSString *PTCONTROLAutomationFollowsEdit;
extern NSString *PTCONTROLMainCounter;
extern NSString *PTCONTROLView;
extern NSString *PTCONTROLDisplayError;
extern NSString *kPTCURSORLEFT;
extern NSString *kPTRETURN;

extern NSString *kTRANSPORTVIEWCLUSTER;
extern NSString *kGROUPLIST;
extern NSString *kPTREGIONLIST;
extern NSString *kTRACKLIST;

extern NSDictionary *allKeyCodes;
extern NSDictionary *allCharCodes;

extern BOOL PTCONTROLKeyDown;
extern BOOL PTCONTROLKeyUp;

#define kPTCHECKMARK [NSString stringWithFormat:@"%C", 0x2713]
#define KEVENTDELAY 10 //WAS 10000

#define releaseRef(theRef)\
if (theRef != nil) { \
	CFRelease(theRef);\
}

#define isRefOk(theRef) \
if (theRef == nil) { \
goto bail;\
}

#define checkErrAndValue(err,val)\
if (err != 0 || !val) {\
goto bail;\
}

#define ConvertName(name) [NSString stringWithFormat:@"%s", #name]

#define checkOrThrowAlert(appRef)\
if (!appRef) {\
(void)NSRunAlertPanel([NSString stringWithFormat:@"The reference was nil %@", ConvertName(appRef)], NSStringFromSelector(_cmd), @"OK", nil, nil);;\
}

enum{
	PTControllerFormatTimeCode = 0,
	PTControllerFormatFeetAndFrames = 1
};

@interface PTController : NSObject
{
}

+ (NSString*)currentProToolsVersion;
+ (NSString*)currentProToolsVersionBundleIdentifier;
+ (NSString*)bundleIdentifierForRunningProtools;
+ (int)currentProToolsVersionInteger;

+ (BOOL)canAXUIProToolsTimeCaptureFromIdentifier:(NSString*)bundleIdentifier andSupplyInfo:(NSDictionary**)dict;
+ (NSDictionary*)AXUIProToolsTimeCapture:(NSString*)bundleIdentifier;

+ (BOOL)canProtoolsCapture:(NSString*)bundleIdentifier;
+ (BOOL)isProtoolsConformReady:(NSString*)bundleIdentifier;
+ (BOOL)isProtoolsConformReadyWithRef:(AXUIElementRef)appRef;
+ (BOOL)isAccessibilityTurnedOn;

+ (BOOL)sendEvent:(NSDictionary*)event ToAppRef:(AXUIElementRef)appRef withOptions:(NSDictionary*)options;
+ (void)sendPasteBoard:(NSString*)theString;

+ (NSString*)currentDisplayModeForBundleIdentifier:(NSString*)bundleIdentifier;
+ (NSString*)currentDisplayModeForAppRef:(AXUIElementRef)appRef;
+ (NSString*)currentDisplayModeForMenuRef:(AXUIElementRef)menuRef;

+ (BOOL)isDocumentAvailable:(AXUIElementRef)appRef;
+ (BOOL)isDocumentInEditMode:(AXUIElementRef)appRef;
+ (BOOL)isAutomationFollowsEditEnabled:(AXUIElementRef)appRef;
+ (BOOL)isVideoTrackEnabledEnabled:(AXUIElementRef)appRef;
+ (BOOL)pressViewMainCounterMenuItemForBundleIdentifier:(NSString*)bundleIdentifier withTitle:(NSString*)title;
+ (void)pressViewMainCounterMenuItemForAppRef:(AXUIElementRef)appRef withTitle:(NSString*)title;
+ (void)pressWindowEditMenuItem:(AXUIElementRef)appRef;
+ (void)pressOptionsAutomationFollowsEditMenuItem:(AXUIElementRef)appRef;
+ (void)pressVideoTrackOnlineMenuItem:(AXUIElementRef)appRef;

+ (BOOL)canPingCopyRefForAppRef:(AXUIElementRef)appRef;
+ (void)pingCopyRefForAppRef:(AXUIElementRef)appRef;
+ (BOOL)isAXUIElementReferenceChecked:(AXUIElementRef)ref;

#pragma mark -
#pragma mark Protools Introspection
#pragma mark -

//TRACKS
+ (NSArray*)allToolBarsFromRefArray:(NSArray*)arrayOfRefs;
+ (NSArray*)allKindsOfTracksForProtoolsAppRef:(AXUIElementRef)appRef;
+ (NSArray*)audioTracksForProtoolsAppRef:(AXUIElementRef)appRef;

//TRACKLISTTRACKS
+ (NSArray*)trackListTracksForProtoolsAppRef:(AXUIElementRef)appRef;
+ (NSArray*)audioTrackListTracksForProtoolsAppRef:(AXUIElementRef)appRef;

+ (NSArray*)allTracksFromRefArray:(NSArray*)arrayOfRefs;
+ (NSArray*)allAudioTracksFromRefArray:(NSArray*)arrayOfRefs;
+ (NSArray*)allKindsOfTracksFromRefArray:(NSArray *)arrayOfRefs;

//GROUPS
+ (AXUIElementRef)groupListToolBarFromAppRef:(AXUIElementRef)appRef;
+ (NSArray*)groupListGroupsForProtoolsAppRef:(AXUIElementRef)appRef;

//REGIONS
+ (AXUIElementRef)regionListToolBarFromAppRef:(AXUIElementRef)appRef;
+ (NSArray*)regionsFromProtoolsAppRef:(AXUIElementRef)appRef;

#pragma mark -
#pragma mark Convenience Methods
#pragma mark -

//has a release issue
+ (AXUIElementRef)protoolsAppRef;

+ (AXUIElementRef)newProtoolsAppRef;
+ (AXUIElementRef)protoolsEditWindowForAppRef:(AXUIElementRef)appRef;
+ (AXUIElementRef)showHideTrackListButton:(AXUIElementRef)appRef;
+ (AXUIElementRef)showHideRegionListButton:(AXUIElementRef)appRef;
+ (AXUIElementRef)trackListPopUpButton:(AXUIElementRef)appRef;
+ (AXUIElementRef)groupListPopUpButton:(AXUIElementRef)appRef;
+ (AXUIElementRef)regionListPopUpButton:(AXUIElementRef)appRef;
+ (AXUIElementRef)editWindowViewSelectorButton:(AXUIElementRef)appRef;

#pragma mark -
#pragma mark Protools 9
#pragma mark -

+ (AXUIElementRef)createReferenceForEditSelectionInButtonFromAppRef:(AXUIElementRef)appRef;
+ (AXUIElementRef)createReferenceForCounterDisplayClusterFromAppRef:(AXUIElementRef)appRef;
+ (AXUIElementRef)createReferenceForTransportViewClusterFromAppRef:(AXUIElementRef)appRef;
+ (AXUIElementRef)createReferenceForGroupToolBarFromAppRef:(AXUIElementRef)appRef;
+ (AXUIElementRef)createReferenceForTrackListToolBarFromAppRef:(AXUIElementRef)appRef;
+ (AXUIElementRef)createReferenceForMainWindowFromAppRef:(AXUIElementRef)appRef;
+ (NSDictionary*)performAction:(NSString*)action OnElement:(AXUIElementRef)elementRef;

+ (CGPoint)scrollPointForGroupToolBarRef;
+ (CGPoint)scrollPointForTrackListToolBarRef;

#pragma mark -
#pragma mark utility
#pragma mark -

+ (NSArray*)allAttributeNamesForAXUIElementRef:(AXUIElementRef)ref;
+ (void)displayEditableAttributesForRef:(AXUIElementRef)ref;

+ (AXUIElementRef)createChildReferenceForCopyMenuItemfromAppRef:(AXUIElementRef)appRef;
+ (AXUIElementRef)createChildReferenceForEditWindowfromAppRef:(AXUIElementRef)appRef;
+ (AXUIElementRef)createChildReferenceForOptionsAutomationFollowsEditfromAppRef:(AXUIElementRef)appRef;
+ (AXUIElementRef)createChildReferenceForAttribute:(NSString*)attr withValue:(NSString*)val fromAppRef:(AXUIElementRef)appRef;
+ (AXUIElementRef)createChildReferenceForVideoTrackEnabledfromAppRef:(AXUIElementRef)appRef;
+ (AXUIElementRef)createChildReferenceForMainMenuCounterFromAppRef:(AXUIElementRef)appRef;
+ (AXUIElementRef)createChildReferenceForMainMenuCounterTitle:(NSString*)title FromAppRef:(AXUIElementRef)appRef;

+ (BOOL)isApplicationIdentifierFrontMost:(NSString*)bundleIdentifier;
+ (BOOL)isApplicationIdentifierRunning:(NSString*)bundleIdentifier;

+ (NSArray*)arrayOfChildrenInfoForAXUIRef:(AXUIElementRef)theRef;

+ (NSMutableDictionary*)allInfoForAXUIElementRef:(AXUIElementRef)theRef;
+ (NSArray*)allInfoForAXUIElementRefArray:(NSArray*)arrayOfRefs;
+ (NSDictionary*)attributeNamesAndValuesForAXUIElementRef:(AXUIElementRef)theRef;
+ (NSDictionary*)actionNamesForAXUIElementRef:(AXUIElementRef)theRef;
+ (BOOL)canRef:(AXUIElementRef)theRef performAction:(NSString*)action;
+ (AXUIElementRef)searchAXUIElement:(AXUIElementRef)parentRef forRefWithAXTitle:(NSString*)title strict:(BOOL)val;

+ (AXUIElementRef)createChildWithAXRole:(NSString*)name forAXUIRef:(AXUIElementRef)theRef;
+ (AXUIElementRef)createChildForAXUIElementRef:(AXUIElementRef)theRef ofName:(NSString*)name forAttribute:(NSString*)attribute strictSearch:(BOOL)value;

//dont use this one as it has a release issue
+ (AXUIElementRef)childForAXUIElementRef:(AXUIElementRef)theRef ofName:(NSString*)name forAttribute:(NSString*)attribute strictSearch:(BOOL)value;

+ (BOOL)isReferenceChecked:(AXUIElementRef)theRef;

+ (CGRect)rectForElement:(AXUIElementRef)element;

#pragma mark -
#pragma mark -


+ (pid_t)retrieveProcessIdForBundleIdentifier:(NSString*)bundleIdentfier;
+ (BOOL)launchApplicationWithBundleIdentifier:(NSString*)bundleIdentifier;
+ (NSArray*)childrenForAXUIElementRef:(AXUIElementRef)theRef;
+ (AXUIElementRef)createApplicationReferenceForBundleIdentifier:(NSString*)bundleIdentfier;

+ (AXUIElementRef)createDialogWindowRefForApplicationIdentfier:(NSString*)bundleIdentfier;

+ (BOOL)nonTitledDialogWindowWasFoundForApplicationIdentfier:(NSString*)bundleIdentfier;
+ (BOOL)nonTitledDialogWindowWasFoundForAppRef:(AXUIElementRef)appRef;

+ (NSArray *)windowRefsForAppRef:(AXUIElementRef)appRef;
+ (NSArray *)windowRefsForApplicationIdentifier:(NSString*)bundleIdentfier;

+ (int)countOfWindowsForApplicationIdentifier:(NSString*)bundleIdentfier;

#pragma mark -
#pragma mark protools specific key combos
#pragma mark -

+ (NSArray*)arrayOfTimeValueDictsForTimeValue:(NSString*)timeValAsString;
+ (NSArray*)arrayOfTimeValuesStrings;

+ (void)sendTimeValueToProtools:(NSString*)timeValAsString toApplicationReference:(AXUIElementRef)appRef;


+ (void)sendComboKeyDict:(NSDictionary*)keyDict toApplicationReference:(AXUIElementRef)appRef withVerbosity:(BOOL)verbose;
+ (NSDictionary *)pauseDict;
+ (NSDictionary *)spaceBarDict;
+ (NSDictionary *)togglePictureDict;
+ (NSDictionary *)slashKeyDict;
+ (NSDictionary *)shuffleKeyDict;
+ (NSDictionary *)slipKeyDict;

+ (NSDictionary *)spotKeyDict;
+ (NSDictionary *)gridKeyDict;

+ (NSDictionary *)returnToZeroDict;
+ (NSDictionary *)markerDict;
+ (NSDictionary *)rightArrowDict;
+ (NSDictionary *)copyDict;
+ (NSDictionary *)cutDict;
+ (NSDictionary *)pasteDict;
+ (NSDictionary *)pasteDictWithString:(NSString*)theValue;
+ (NSDictionary *)seperateRegionAtSelectionDict;
+ (NSDictionary *)healDict;
+ (NSDictionary *)reselectAllShiftDownDict;
+ (NSDictionary *)reselectAllShiftUpDict;
+ (NSDictionary *)insertSilenceDict;
+ (NSDictionary *)shiftAppleAdvanceDict;
+ (NSDictionary *)optionAppleDecrementDict;
+ (NSDictionary *)plusAdvanceDict;
+ (NSDictionary *)minusAdvanceDict;
+ (NSDictionary *)numericDictForNumber:(NSString*)stringNum;


#pragma mark -
#pragma mark protools specific key utilities
#pragma mark -

+ (NSDictionary *)keyComboDictWithValues:(NSArray *)vals;
+ (NSArray *)keyComboKeys;

#pragma mark -
#pragma mark key & char code info
#pragma mark -


+ (NSDictionary*)globalKeyCodes;
+ (NSDictionary*)globalCharCodes;

+ (NSDictionary *)keyCodes;
+ (NSDictionary *)charCodes;
+ (int)keyPadKeyCodeForInteger:(int)numAsInt;


#pragma -
#pragma Mouse Events
#pragma -

+ (CGPoint)currentMouseLocation;
+ (void)moveMouseToPosition:(CGPoint)point;
+ (void)singleClickPosition:(CGPoint)point 
                  withShift:(BOOL)useShift
                withControl:(BOOL)useControl
                 withOption:(BOOL)useOption
                withCommand:(BOOL)useCommand;

@end
